//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "VVetoPerf.h"

////////////////////////////////////////////////////////////////////////////////////
bool VetoPerf::MakeReport(void){
////////////////////////////////////////////////////////////////////////////////////

  stringstream sstmp;
  
  // sorted veto indexes
  vector <int> Vindexes=Sort();

  // import material
  system(("cp -f ${VETOPERF_HTML}/style.css "+outdir+"/style.css").c_str());
  system(("cp -f ${GWOLLUM_DOC}/Pics/gwollum_logo_min_trans.gif "+outdir+"/icon.gif").c_str());

  // index header & scripts
  ofstream report((outdir+"/index.html").c_str());
  report<<"<html>"<<endl;
  report<<"<head>"<<endl;
  report<<"<title>VetoPerf Report</title>"<<endl;
  report<<"<link rel=\"stylesheet\" href=\"style.css\" type=\"text/css\" />"<<endl;
  report<<"<link rel=\"icon\" type=\"image/x-icon\" href=\"./icon.gif\" />"<<endl;
  report<<"<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"./icon.gif\" />"<<endl;
  report<<"<script type=\"text/javascript\">"<<endl;
  report<<"function showPlot(target, source, type, allp) {"<<endl;
  report<<"  var basename ="<<endl;
  report<<"    source + \"_\" + type;"<<endl;
  report<<"  if (allp==\"all\") {"<<endl;
  report<<"    document.getElementById(\"plot_\" + source).src ="<<endl;
  report<<"    \"./\" + target + \"-all_\" + type +\".png\";"<<endl;
  report<<"  }"<<endl;
  report<<"  else {"<<endl;
  report<<"    document.getElementById(\"plot_\" + source).src ="<<endl;
  report<<"    \"./\" + target + \"-\" + source + \"_\" + type +\".png\";"<<endl;
  report<<"  }"<<endl;
  report<<"}"<<endl;
  report<<"</script>"<<endl;
  report<<"<script type=\"text/javascript\">"<<endl;
  report<<"function toggle(anId) {"<<endl;
  report<<"  node = document.getElementById(anId);"<<endl;
  report<<"  if (node.style.visibility==\"hidden\") {"<<endl;
  report<<"    node.style.visibility = \"visible\";"<<endl;
  report<<"    node.style.height = \"auto\";"<<endl;
  report<<"  }"<<endl;
  report<<"  else {"<<endl;
  report<<"    node.style.visibility = \"hidden\";"<<endl;
  report<<"    node.style.height = \"0\";"<<endl;
  report<<"  }"<<endl;
  report<<"}"<<endl;
  report<<"</script>"<<endl;
  report<<"</head>"<<endl;
  report<<"<body>"<<endl;
  report<<endl;

  // index title
  report<<"<h1>VetoPerf analysis over "<<TP->GetName()<<" </h1>"<<endl;
  report<<"<hr />"<<endl;
  report<<endl;

  if(subtitle.compare("")){
    report<<"<pre>"<<subtitle<<"</pre>"<<endl;
    report<<"<hr />"<<endl;
    report<<endl;
  }

  // index summary
  report<<"<h2>Summary</h2>"<<endl;
  tm utc;
  time_t timer_stop;
  time ( &timer_stop );
  struct tm * ptm = gmtime ( &timer_stop );
  report<<"<table>"<<endl;
  report<<"  <tr><td>VetoPerf run by:</td><td>"<<getenv("USER")<<"</td></tr>"<<endl;
  report<<"  <tr><td>VetoPerf processing time:</td><td>"<<(int)(timer_stop-timer_start)/3600<<"h, "<<((int)(timer_stop-timer_start)%3600)/60<<"min</td></tr>"<<endl;
  report<<"  <tr><td>Processing Date:</td><td>"<<asctime(ptm)<<" (UTC)</td></tr>"<<endl;
  GPSToUTC (&utc, InSeg->GetStart(0));
  report<<"  <tr><td>Requested start:</td><td>"<<(int)InSeg->GetStart(0)<<" &rarr; "<<asctime(&utc)<<" (UTC)</td></tr>"<<endl;
  GPSToUTC (&utc, InSeg->GetEnd(InSeg->GetNsegments()-1));
  report<<"  <tr><td>Requested stop:</td><td>"<<(int)InSeg->GetEnd(InSeg->GetNsegments()-1)<<" &rarr; "<<asctime(&utc)<<" (UTC)</td></tr>"<<endl;
  report<<"  <tr><td>Requested livetime:</td><td>"<<(int)InSeg->GetLiveTime()<<" sec &rarr; "<<setprecision(3)<<fixed<<InSeg->GetLiveTime()/3600.0/24<<" days</td></tr>"<<endl;
  InSeg->Write(outdir+"/vp.insegments.txt");
  report<<"  <tr><td>Requested segments:</td><td><a href=\"./vp.insegments.txt\">vp.insegments.txt</a></td></tr>"<<endl;
  report<<"</table>"<<endl;
  report<<"<hr />"<<endl;
  report<<endl;

  // triggers summary
  report<<"<h2>Triggers ("<<TP->GetName()<<" - "<<TP->GetProcessName()<<")</h2>"<<endl;
  report<<"<table>"<<endl;
  report<<"  <tr><td>Number of raw triggers:</td><td>"<<TP->GetNtriggers()<<"</td></tr>"<<endl;
  report<<"  <tr><td>Number of raw clusters:</td><td>"<<TP->GetNClusters()<<"</td></tr>"<<endl;
  if(freqmin>TP->GetFrequencyMin()) report<<"  <tr><td>Minimum frequency:</td><td>"<<freqmin<<" Hz</td></tr>"<<endl;
  if(freqmax<TP->GetFrequencyMax()) report<<"  <tr><td>Maximum frequency:</td><td>"<<freqmax<<" Hz</td></tr>"<<endl;
  report<<"  <tr><td>Number of active clusters:</td><td>"<<n_c<<"</td></tr>"<<endl;
  report<<"  <tr><td>Number of active clusters:</td><td>";
  for(int s=0; s<(int)snrthr.size(); s++) report<<n_c_snr[s]<<" (SNR &gt; "<<snrthr[s]<<") ";
  report<<"</td></tr>"<<endl;
  report<<"</table>"<<endl;
  report<<"<hr />"<<endl;

  // veto summary
  report<<"<h2>Vetoes</h2>"<<endl;
  report<<"<table>"<<endl;
  report<<"  <tr><td>Number of vetoes:</td><td>"<<nveto<<"</td></tr>"<<endl;
  report<<"</table>"<<endl;
  report<<"<hr />"<<endl;
  report<<endl;

  // loop over vetoes
  double sig_max;
  string color_effdead;
  for(int v=0; v<nveto; v++){

    // copy veto file
    sstmp<<"cp -f "<<vp[Vindexes[v]]->vetofile<<" "<<outdir<<"/vp.v"<<v<<".txt";
    if(!vp[Vindexes[v]]->vetochannel) system(sstmp.str().c_str());
    sstmp.clear(); sstmp.str("");

    // get best efficiency/dead-time value (ranking)
    sig_max=0.0;
    for(int s=0; s<(int)snrthr.size(); s++)
      if((double)(vp[Vindexes[v]]->n_cveto[s])*InSeg->GetLiveTime()/vp[Vindexes[v]]->dt/(double)(n_c_snr[s])>sig_max)
	sig_max=(double)(vp[Vindexes[v]]->n_cveto[s])*InSeg->GetLiveTime()/vp[Vindexes[v]]->dt/(double)(n_c_snr[s]);
    if(sig_max>100) color_effdead="red";
    else if(sig_max>10) color_effdead="orange";
    else if(sig_max>2) color_effdead="green";
    else color_effdead="black";

    // veto header
    report<<"<h2><a name=\"V"<<v<<"\"></a><font color=\"red\">V"<<v<<":</font> "<<vp[Vindexes[v]]->vetoname<<" (&epsilon; = "<<(double)(vp[Vindexes[v]]->n_cveto[0])/(double)n_c_snr[0]*100.0<<"%, <font color=\""<<color_effdead<<"\">&epsilon;/d = "<<sig_max<<"</font>) <a href=\"javascript:void(0)\" name=\""<<vp[Vindexes[v]]->vetoname<<"\" onclick=\"toggle('src_"<<vp[Vindexes[v]]->vetoname<<"')\">[click here to expand/hide]</a> <a href=\"#V"<<v<<"\">[link to here]</a></h2>"<<endl;
    report<<"<div id=\"src_"<<vp[Vindexes[v]]->vetoname<<"\" style=\"visibility:hidden;height:0;\">"<<endl;
    report<<"  <table>"<<endl;
    report<<"    <tr>"<<endl;

    report<<"      <td>"<<endl;

    // input section
    report<<"        <table class=\"list\">"<<endl;
    sstmp<<"./vp.v"<<v<<".txt";
    if(!vp[Vindexes[v]]->vetochannel) report<<"          <tr><td><a href=\""<<sstmp.str()<<"\">Segment file</a></td></tr>"<<endl;
    sstmp.clear(); sstmp.str("");
    report<<"        </table>"<<endl;

    // veto segments section
    report<<"        <table class=\"list\">"<<endl;
    report<<"          <tr><td><b>Veto:</b></td><td></td><td></td></tr>"<<endl;
    report<<"          <tr><td>Number of veto segments:</td><td>"<<vp[Vindexes[v]]->n_seg<<"</td><td></td></tr>"<<endl;
    report<<"          <tr><td>Veto dead-time:</td><td>"<<vp[Vindexes[v]]->dt<<" s</td><td>("<<vp[Vindexes[v]]->dt/InSeg->GetLiveTime()*100.0<<" %)</td></tr>"<<endl;
    report<<"        </table>"<<endl;

    // performance section
    report<<"        <table class=\"list\">"<<endl;
    report<<"          <tr><td><b>Performance:</b></td></tr>"<<endl;
    report<<"        </table>"<<endl;

    report<<"        <table class=\"perflist\">"<<endl;
    report<<"          <tr>"<<endl;
    report<<"            <th></th>"<<endl;
    for(int s=0; s<(int)snrthr.size(); s++) report<<"            <th colspan=\"2\">SNR &gt; "<<snrthr[s]<<"</th>"<<endl;
    report<<"          </tr>"<<endl;

    report<<"          <tr>"<<endl;
    report<<"            <td>Number of vetoed clusters:</td>"<<endl;
    for(int s=0; s<(int)snrthr.size(); s++) report<<"            <td align=\"right\">"<<vp[Vindexes[v]]->n_cveto[s]<<"/"<<n_c_snr[s]<<"</td><td align=\"right\">("<<(double)(vp[Vindexes[v]]->n_cveto[s])/(double)n_c_snr[s]*100.0<<"%)</td>"<<endl;
    report<<"          </tr>"<<endl;

    report<<"          <tr>"<<endl;
    report<<"            <td>Number of used segments:</td>"<<endl;
    for(int s=0; s<(int)snrthr.size(); s++) report<<"            <td>"<<vp[Vindexes[v]]->n_seguse[s]<<"</td><td>("<<(double)(vp[Vindexes[v]]->n_seguse[s])/(double)(vp[Vindexes[v]]->n_seg)*100.0<<"%)</td>"<<endl;
    report<<"          </tr>"<<endl;

    report<<"        </table>"<<endl;

    // Plot section
    report<<"        <table class=\"list\">"<<endl;
    report<<"          <tr><td><b>Plots:</b></td><td></td></tr>"<<endl;

    // SNR plot
    report<<"          <tr><td>&rarr; <a href=\"javascript:showPlot('"<<TP->GetNameConv()<<"', '"<<vp[Vindexes[v]]->vetoname<<"', 'snr','toto');\">SNR distribution</a></td><td></td></tr>"<<endl;

    // time-frequency plot
    report<<"          <tr><td>&rarr; Time-frequency trigger distribution:</td><td><a href=\"javascript:showPlot('"<<TP->GetNameConv()<<"', '"<<vp[Vindexes[v]]->vetoname<<"', 'freqtime','all');\">before</a>/<a href=\"javascript:showPlot('"<<TP->GetNameConv()<<"', '"<<vp[Vindexes[v]]->vetoname<<"', 'freqtime','toto');\">after</a> veto</td></tr>"<<endl;

    // SNR-time
    report<<"          <tr><td>&rarr; SNR-time trigger distribution:</td><td><a href=\"javascript:showPlot('"<<TP->GetNameConv()<<"', '"<<vp[Vindexes[v]]->vetoname<<"', 'snrtime','all');\">before</a>/<a href=\"javascript:showPlot('"<<TP->GetNameConv()<<"', '"<<vp[Vindexes[v]]->vetoname<<"', 'snrtime','toto');\">after</a> veto</td></tr>"<<endl;

    // SNR-frequency
    report<<"          <tr><td>&rarr; SNR-frequency trigger distribution:</td><td><a href=\"javascript:showPlot('"<<TP->GetNameConv()<<"', '"<<vp[Vindexes[v]]->vetoname<<"', 'snrfreq','all');\">before</a>/<a href=\"javascript:showPlot('"<<TP->GetNameConv()<<"', '"<<vp[Vindexes[v]]->vetoname<<"', 'snrfreq','toto');\">after</a> veto</td></tr>"<<endl;

    report<<"        </table>"<<endl;
    report<<"      </td>"<<endl;

    // plot area
    report<<"      <td>"<<endl;
    report<<"        <img id=\"plot_"<<vp[Vindexes[v]]->vetoname<<"\" src=\"./"<<TP->GetNameConv()<<"-"<<vp[Vindexes[v]]->vetoname<<"_snr.png\" />"<<endl;
    report<<"      </td>"<<endl;

    report<<"    </tr>"<<endl;
    report<<"  </table>"<<endl;
    report<<"</div>"<<endl;
    report<<"<hr/>"<<endl;
    report<<endl;

  }
  
  // index footer
  report<<"<table>"<<endl;
  report<<"  <tr><td>Florent Robinet, <a href=\"mailto:robinet@lal.in2p3.fr\">robinet@lal.in2p3.fr</a></td></tr>"<<endl;
  report<<"</table>"<<endl;
  report<<"</body>"<<endl;
  report<<"</html>"<<endl;
  report.close();

  return true;
}

////////////////////////////////////////////////////////////////////////////////////
vector <int> VetoPerf::Sort(void){
////////////////////////////////////////////////////////////////////////////////////
  vector <int> s_index;// sorted indexes

  if(!nveto) return s_index;

  vector <double> s_sig;
  bool sorted;

  // init
  s_index.push_back(0);
  s_sig.push_back(vp[0]->n_cveto[0]);

  for(int v=1; v<nveto; v++){
    sorted=false;

    for(int i=0; i<(int)s_index.size(); i++){
      if(vp[v]->n_cveto[0]>=s_sig[i]){
	s_index.insert(s_index.begin()+i,v);
	s_sig.insert(s_sig.begin()+i,vp[v]->n_cveto[0]);
	sorted=true;
	break;
      }
    }
    if(!sorted){
      s_index.push_back(v);
      s_sig.push_back(vp[v]->n_cveto[0]);
    }
  }
  
  s_sig.clear();

  return s_index;
}
